@@ -10,7 +10,7 @@ NULL
 #' @param post An \eqn{n} x \eqn{1} vector of Post-Treatment dummies (post = 1 if observation belongs to post-treatment period,
 #'             and post = 0 if observation belongs to pre-treatment period.)
 #' @param D An \eqn{n} x \eqn{1} vector of Group indicators (=1 if observation is treated in the post-treatment, =0 otherwise).
-#' @param covariates An \eqn{n} x \eqn{k} matrix of covariates to be used in the propensity score and regression estimation.
+#' @param covariates An \eqn{n} x \eqn{k} matrix of covariates to be used in the propensity score and regression estimation. Please add a vector of constants if you want to include an intercept in the models.
 #' If covariates = NULL, this leads to an unconditional DiD estimator.
 #' @param i.weights An \eqn{n} x \eqn{1} vector of weights to be used. If NULL, then every observation has the same weights. The weights are normalized and therefore enforced to have mean 1 across all observations.
 #' @param boot Logical argument to whether bootstrap should be used for inference. Default is FALSE.
@@ -52,7 +52,7 @@ NULL
 #'
 #' @examples
 #' # use the simulated data provided in the package
-#' covX = as.matrix(sim_rc[,5:8])
+#' covX = as.matrix(cbind(1, sim_rc[,5:8]))
 #' # Implement the 'traditional' locally efficient DR DiD estimator
 #' drdid_rc(y = sim_rc$y, post = sim_rc$post, D = sim_rc$d,
 #'          covariates= covX)
@@ -71,14 +71,11 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   y <- as.vector(y)
   # post as vector
   post <- as.vector(post)
-  # Add constant to covariate vector
-  int.cov <- as.matrix(rep(1,n))
-  if (!is.null(covariates)){
-    if(all(as.matrix(covariates)[,1]==rep(1,n))){
-      int.cov <- as.matrix(covariates)
-    } else {
-      int.cov <- as.matrix(cbind(1, covariates))
-    }
+  # Covariate vector
+  if(is.null(covariates)){
+    int.cov <- as.matrix(rep(1,n))
+  } else{
+    int.cov <- as.matrix(covariates)
   }
 
   # Weights
@@ -89,22 +86,28 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   # Normalize weights
   i.weights <- i.weights/mean(i.weights)
   #-----------------------------------------------------------------------------
-  #Compute the Pscore by MLE
-  #pscore.tr <- stats::glm(D ~ -1 + int.cov, family = "binomial", weights = i.weights)
-  pscore.tr <- suppressWarnings(parglm::parglm(D ~ -1 + int.cov, family = "binomial", weights = i.weights))
+  # Compute the Pscore by MLE
+  pscore.tr <- suppressWarnings(fastglm::fastglm(
+                                          x = int.cov,
+                                          y = D,
+                                          family = stats::binomial(),
+                                          weights = i.weights,
+                                          intercept = FALSE,
+                                          method = 3
+  ))
+
+  class(pscore.tr) <- "glm" #this allow us to use vcov
   if(pscore.tr$converged == FALSE){
     warning(" glm algorithm did not converge")
   }
   if(anyNA(pscore.tr$coefficients)){
     stop("Propensity score model coefficients have NA components. \n Multicollinearity (or lack of variation) of covariates is a likely reason.")
   }
-  ps.fit <- as.vector(pscore.tr$fitted.values)
+  ps.fit <- fitted(pscore.tr)
   # Avoid divide by zero
   ps.fit <- pmin(ps.fit, 1 - 1e-6)
-  #Compute the Outcome regression for the control group at the pre-treatment period, using ols.
-  # reg.cont.coeff.pre <- stats::coef(stats::lm(y ~ -1 + int.cov,
-  #                                             subset = ((D==0) & (post==0)),
-  #                                             weights = i.weights))
+  W <- ps.fit * (1 - ps.fit) * i.weights
+  # Compute the Outcome regression for the control group at the pre-treatment period, using ols.
   pre_filter <- (D == 0) & (post == 0)
   reg.cont.coeff.pre <- stats::coef(fastglm::fastglm(
                                     x = int.cov[pre_filter, , drop = FALSE],
@@ -116,10 +119,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
     stop("Outcome regression model coefficients have NA components. \n Multicollinearity (or lack of variation) of covariates is a likely reason.")
   }
   out.y.cont.pre <-   as.vector(tcrossprod(reg.cont.coeff.pre, int.cov))
-  #Compute the Outcome regression for the control group at the post-treatment period, using ols.
-  # reg.cont.coeff.post <- stats::coef(stats::lm(y ~ -1 + int.cov,
-  #                                              subset = ((D==0) & (post==1)),
-  #                                              weights = i.weights))
+  # Compute the Outcome regression for the control group at the post-treatment period, using ols.
 
   post_filter <- (D == 0) & (post == 1)
   reg.cont.coeff.post <- stats::coef(fastglm::fastglm(
@@ -136,10 +136,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   out.y.cont <- post * out.y.cont.post + (1 - post) * out.y.cont.pre
 
 
-  #Compute the Outcome regression for the treated group at the pre-treatment period, using ols.
-  # reg.treat.coeff.pre <- stats::coef(stats::lm(y ~ -1 + int.cov,
-  #                                              subset = ((D==1) & (post==0)),
-  #                                              weights = i.weights))
+  # Compute the Outcome regression for the treated group at the pre-treatment period, using ols.
   pre_treat_filter <- (D == 1) & (post == 0)
   reg.treat.coeff.pre <- stats::coef(fastglm::fastglm(
                               x = int.cov[pre_treat_filter, , drop = FALSE],
@@ -148,10 +145,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
                               family = gaussian(link = "identity")
   ))
   out.y.treat.pre <-   as.vector(tcrossprod(reg.treat.coeff.pre, int.cov))
-  #Compute the Outcome regression for the treated group at the post-treatment period, using ols.
-  # reg.treat.coeff.post <- stats::coef(stats::lm(y ~ -1 + int.cov,
-  #                                               subset = ((D==1) & (post==1)),
-  #                                               weights = i.weights))
+  # Compute the Outcome regression for the treated group at the post-treatment period, using ols.
   post_treat_filter <- (D == 1) & (post == 1)
   reg.treat.coeff.post <- stats::coef(fastglm::fastglm(
                                       x = int.cov[post_treat_filter, , drop = FALSE],
@@ -161,7 +155,6 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   ))
   out.y.treat.post <-   as.vector(tcrossprod(reg.treat.coeff.post, int.cov))
 
-
   #-----------------------------------------------------------------------------
   # First, the weights
   w.treat.pre <- i.weights * D * (1 - post)
@@ -211,6 +204,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   weights.ols.pre <- i.weights * (1 - D) * (1 - post)
   wols.x.pre <- weights.ols.pre * int.cov
   wols.eX.pre <- weights.ols.pre * (y - out.y.cont.pre) * int.cov
+  #XpX_pre <- opt_crossprod(wols.x.pre, int.cov, n)
   XpX_pre <- base::crossprod(wols.x.pre, int.cov)/n
   # Check if XpX is invertible
   if ( base::rcond(XpX_pre) < .Machine$double.eps) {
@@ -223,6 +217,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   weights.ols.post <- i.weights * (1 - D) * post
   wols.x.post <- weights.ols.post * int.cov
   wols.eX.post <- weights.ols.post * (y - out.y.cont.post) * int.cov
+  #XpX_post <- opt_crossprod(wols.x.post, int.cov, n)
   XpX_post <- base::crossprod(wols.x.post, int.cov)/n
   # Check if XpX is invertible
   if ( base::rcond(XpX_post) < .Machine$double.eps) {
@@ -235,6 +230,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   weights.ols.pre.treat <- i.weights * D * (1 - post)
   wols.x.pre.treat <- weights.ols.pre.treat * int.cov
   wols.eX.pre.treat <- weights.ols.pre.treat * (y - out.y.treat.pre) * int.cov
+  #XpX_pre_treat <- opt_crossprod(wols.x.pre.treat, int.cov, n)
   XpX_pre_treat <- base::crossprod(wols.x.pre.treat, int.cov)/n
   # Check if XpX is invertible
   if ( base::rcond(XpX_pre_treat) < .Machine$double.eps) {
@@ -248,6 +244,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   weights.ols.post.treat <- i.weights * D *  post
   wols.x.post.treat <- weights.ols.post.treat * int.cov
   wols.eX.post.treat <- weights.ols.post.treat * (y - out.y.treat.post) * int.cov
+  #XpX_post_treat <- opt_crossprod(wols.x.post.treat, int.cov, n)
   XpX_post_treat <- base::crossprod(wols.x.post.treat, int.cov)/n
   # Check if XpX is invertible
   if ( base::rcond(XpX_post_treat) < .Machine$double.eps) {
@@ -258,7 +255,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
 
   # Asymptotic linear representation of logit's beta's
   score.ps <- i.weights * (D - ps.fit) * int.cov
-  Hessian.ps <- stats::vcov(pscore.tr) * n
+  Hessian.ps <- chol2inv(chol(t(int.cov) %*% (W * int.cov))) * n
   asy.lin.rep.ps <-  score.ps %*% Hessian.ps
   #-----------------------------------------------------------------------------
   # Now, the influence function of the "treat" component
@@ -274,10 +271,7 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   # Now get the influence function related to the estimation effect related to beta's
   inf.treat.or.post <- asy.lin.rep.ols.post %*% M1.post
   inf.treat.or.pre <- asy.lin.rep.ols.pre %*% M1.pre
-  inf.treat.or <- inf.treat.or.post + inf.treat.or.pre
 
-  # Influence function for the treated component
-  inf.treat <- inf.treat.post - inf.treat.pre + inf.treat.or
   #-----------------------------------------------------------------------------
   # Now, get the influence function of control component
   # Leading term of the influence function: no estimation effect from nuisance parameters
@@ -299,13 +293,6 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   # Now get the influence function related to the estimation effect related to beta's
   inf.cont.or.post <- asy.lin.rep.ols.post %*% M3.post
   inf.cont.or.pre <- asy.lin.rep.ols.pre %*% M3.pre
-  inf.cont.or <- inf.cont.or.post + inf.cont.or.pre
-
-  # Influence function for the control component
-  inf.cont <- inf.cont.post - inf.cont.pre + inf.cont.ps + inf.cont.or
-  #-----------------------------------------------------------------------------
-  #get the influence function of the inefficient DR estimator (put all pieces together)
-  dr.att.inf.func1 <- inf.treat - inf.cont
   #-----------------------------------------------------------------------------
   # Now, we only need to get the influence function of the adjustment terms
   # First, the terms as if all OR parameters were known
@@ -320,17 +307,52 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
   mom.pre <- base::colMeans((w.d/mean(w.d) -  w.dt0/mean(w.dt0)) * int.cov)
   inf.or.post <- (asy.lin.rep.ols.post.treat - asy.lin.rep.ols.post) %*% mom.post
   inf.or.pre <-  (asy.lin.rep.ols.pre.treat - asy.lin.rep.ols.pre) %*% mom.pre
+
+  # # Pre case
+  # batch_results_pre <- batch_matrix_operations_rc(asy.lin.rep.ps, wols.eX.pre, XpX.inv.pre,
+  #                                            wols.eX.pre.treat, XpX.inv.pre.treat,
+  #                                            M1.pre, M2.pre, M3.pre, mom.pre)
+  #
+  # # Post case
+  # batch_results_post <- batch_matrix_operations_rc(asy.lin.rep.ps, wols.eX.post, XpX.inv.post,
+  #                                             wols.eX.post.treat, XpX.inv.post.treat,
+  #                                             M1.post, M2.post, M3.post, mom.post)
+
+  # # Access the results: pre
+  # inf.cont.ps.pre <- batch_results_pre$inf_cont_ps
+  # inf.treat.or.pre <- batch_results_pre$inf_treat_or
+  # inf.cont.or.pre <- batch_results_pre$inf_cont_or
+  # inf.or.pre <- batch_results_pre$inf_or
+  # # Access the results: post
+  # inf.cont.ps.post <- batch_results_post$inf_cont_ps
+  # inf.treat.or.post <- batch_results_post$inf_treat_or
+  # inf.cont.or.post <- batch_results_post$inf_cont_or
+  # inf.or.post <- batch_results_post$inf_or
+
+  # get the influence function related to the estimation effect related to beta's
+  inf.treat.or <- inf.treat.or.post + inf.treat.or.pre
+  inf.cont.or <- inf.cont.or.post + inf.cont.or.pre
+  # Now the estimation effect of the OR coefficients
   inf.or <- inf.or.post - inf.or.pre
+
+  # Influence function for the treated component
+  inf.treat <- inf.treat.post - inf.treat.pre + inf.treat.or
+
+  # Influence function for the control component
+  inf.cont <- inf.cont.post - inf.cont.pre + inf.cont.ps + inf.cont.or
   #-----------------------------------------------------------------------------
-  #get the influence function of the locally efficient DR estimator (put all pieces together)
+  # get the influence function of the inefficient DR estimator (put all pieces together)
+  dr.att.inf.func1 <- inf.treat - inf.cont
+  #-----------------------------------------------------------------------------
+  # get the influence function of the locally efficient DR estimator (put all pieces together)
   dr.att.inf.func <- dr.att.inf.func1 + inf.eff + inf.or
   #-----------------------------------------------------------------------------
   if (boot == FALSE) {
     # Estimate of standard error
     se.dr.att <- stats::sd(dr.att.inf.func)/sqrt(n)
-    # Estimate of upper boudary of 95% CI
+    # Estimate of upper boundary of 95% CI
     uci <- dr.att + 1.96 * se.dr.att
-    # Estimate of lower doundary of 95% CI
+    # Estimate of lower boundary of 95% CI
     lci <- dr.att - 1.96 * se.dr.att
     #Create this null vector so we can export the bootstrap draws too.
     dr.boot <- NULL
@@ -343,11 +365,11 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
       dr.boot <- mboot.did(dr.att.inf.func, nboot)
       # get bootstrap std errors based on IQR
       se.dr.att <- stats::IQR(dr.boot) / (stats::qnorm(0.75) - stats::qnorm(0.25))
-      # get symmtric critival values
+      # get symmetric critical values
       cv <- stats::quantile(abs(dr.boot/se.dr.att), probs = 0.95)
-      # Estimate of upper boudary of 95% CI
+      # Estimate of upper boundary of 95% CI
       uci <- dr.att + cv * se.dr.att
-      # Estimate of lower doundary of 95% CI
+      # Estimate of lower boundary of 95% CI
       lci <- dr.att - cv * se.dr.att
     } else {
       # do weighted bootstrap
@@ -356,11 +378,11 @@ drdid_rc <-function(y, post, D, covariates, i.weights = NULL,
                                D = D, int.cov = int.cov, i.weights = i.weights))
       # get bootstrap std errors based on IQR
       se.dr.att <- stats::IQR((dr.boot - dr.att)) / (stats::qnorm(0.75) - stats::qnorm(0.25))
-      # get symmtric critival values
+      # get symmetric critical values
       cv <- stats::quantile(abs((dr.boot - dr.att)/se.dr.att), probs = 0.95)
-      # Estimate of upper boudary of 95% CI
+      # Estimate of upper boundary of 95% CI
       uci <- dr.att + cv * se.dr.att
-      # Estimate of lower doundary of 95% CI
+      # Estimate of lower boundary of 95% CI
       lci <- dr.att - cv * se.dr.att
 
     }
