@@ -10,11 +10,11 @@ wboot_drdid_imp_rc1 <- function(nn, n, y, post, D, int.cov, i.weights){
   #Compute the Pscore using the pscore.cal
   ps.b <- pscore.cal(D, int.cov, i.weights = b.weights, n = n)
   ps.b <- as.vector(ps.b$pscore)
-  ps.b <- pmin(ps.b, 1 - 1e-16)
+  ps.b <- pmin(ps.b, 1 - 1e-6)
   #Compute the Outcome regression for the control group
-  out.y.pre.b <- wols_rc(y, post, D, int.cov, ps.b, b.weights, pre = T, treat = F)
+  out.y.pre.b <- wols_rc(y, post, D, int.cov, ps.b, b.weights, pre = TRUE, treat = FALSE)
   out.y.pre.b <-  as.vector(out.y.pre.b$out.reg)
-  out.y.post.b <- wols_rc(y, post, D, int.cov, ps.b, b.weights, pre = F, treat = F)
+  out.y.post.b <- wols_rc(y, post, D, int.cov, ps.b, b.weights, pre = FALSE, treat = FALSE)
   out.y.post.b <-  as.vector(out.y.post.b$out.reg)
 
   # Combine the ORs
