@@ -9,7 +9,12 @@ wboot_std_ipw_rc <- function(nn, n, y, post, D, int.cov, i.weights){
   b.weights <- as.vector(i.weights * v)
   # Propensity score estimation
   # ps.b <- suppressWarnings(stats::glm(D ~ -1 + int.cov, family = "binomial", weights = b.weights)$fitted.values)
-  ps.b <- suppressWarnings(parglm::parglm(D ~ -1 + int.cov, family = "binomial", weights = b.weights)$fitted.values)
+  ps.b <- suppressWarnings(fastglm::fastglm(x = int.cov,
+                                            y = D,
+                                            family = stats::binomial(),
+                                            weights =  b.weights,
+                                            intercept = FALSE,
+                                            method = 3)$fitted.values)
   ps.b <- pmin(ps.b, 1 - 1e-6)
   # Compute  standardized IPW estimator
   w.treat.pre.b <- b.weights * D * (1 - post)
